@@ -4,7 +4,8 @@
 
 from abc import ABC, abstractmethod
 from copy import deepcopy
-from typing import Any, Callable, Dict, Optional, Union, cast
+from typing import Any, Callable, Dict, NewType, Optional, Union, cast
+from warnings import warn
 
 import torch
 from torch import Tensor, ones, optim
@@ -14,32 +15,29 @@ from torch.utils.data.sampler import SubsetRandomSampler
 from torch.utils.tensorboard import SummaryWriter
 
 from sbi import utils as utils
-from sbi.inference import NeuralInference
+from sbi.inference import NeuralInference, check_if_proposal_has_default_x
 from sbi.inference.posteriors.direct_posterior import DirectPosterior
+from sbi.types import TorchModule
 from sbi.utils import (
+    RestrictedPrior,
     check_estimator_arg,
-    x_shape_from_simulation,
     test_posterior_net_for_multi_d_x,
+    validate_theta_and_x,
+    x_shape_from_simulation,
 )
+from sbi.utils.sbiutils import mask_sims_from_prior
 
 
 class PosteriorEstimator(NeuralInference, ABC):
     def __init__(
         self,
-        simulator: Callable,
         prior,
-        num_workers: int = 1,
-        simulation_batch_size: int = 1,
         density_estimator: Union[str, Callable] = "maf",
-        sample_with_mcmc: bool = False,
-        mcmc_method: str = "slice_np",
-        mcmc_parameters: Optional[Dict[str, Any]] = None,
-        rejection_sampling_parameters: Optional[Dict[str, Any]] = None,
         device: str = "cpu",
         logging_level: Union[int, str] = "WARNING",
         summary_writer: Optional[SummaryWriter] = None,
         show_progress_bars: bool = True,
-        show_round_summary: bool = False,
+        **unused_args
     ):
         """Base class for Sequential Neural Posterior Estimation methods.
 
@@ -52,39 +50,20 @@ class PosteriorEstimator(NeuralInference, ABC):
                 needs to return a PyTorch `nn.Module` implementing the density
                 estimator. The density estimator needs to provide the methods
                 `.log_prob` and `.sample()`.
-            sample_with_mcmc: Whether to sample with MCMC. MCMC can be used to deal
-                with high leakage.
-            mcmc_method: Method used for MCMC sampling, one of `slice_np`, `slice`,
-                `hmc`, `nuts`. Currently defaults to `slice_np` for a custom numpy
-                implementation of slice sampling; select `hmc`, `nuts` or `slice` for
-                Pyro-based sampling.
-            mcmc_parameters: Dictionary overriding the default parameters for MCMC.
-                The following parameters are supported: `thin` to set the thinning
-                factor for the chain, `warmup_steps` to set the initial number of
-                samples to discard, `num_chains` for the number of chains,
-                `init_strategy` for the initialisation strategy for chains; `prior` will
-                draw init locations from prior, whereas `sir` will use
-                Sequential-Importance-Resampling using `init_strategy_num_candidates`
-                to find init locations.
-            rejection_sampling_parameters: Dictonary overriding the default parameters
-                for rejection sampling. The following parameters are supported:
-                `max_sampling_batch_size` to set the batch size for drawing new
-                samples from the candidate distribution, e.g., the posterior. Larger
-                batch size speeds up sampling.
+            unused_args: Absorbs additional arguments. No entries will be used. If it
+                is not empty, we warn. In future versions, when the new interface of
+                0.14.0 is more mature, we will remove this argument.
 
         See docstring of `NeuralInference` class for all other arguments.
         """
 
         super().__init__(
-            simulator=simulator,
             prior=prior,
-            num_workers=num_workers,
-            simulation_batch_size=simulation_batch_size,
             device=device,
             logging_level=logging_level,
             summary_writer=summary_writer,
             show_progress_bars=show_progress_bars,
-            show_round_summary=show_round_summary,
+            **unused_args,
         )
 
         # As detailed in the docstring, `density_estimator` is either a string or
@@ -97,22 +76,68 @@ class PosteriorEstimator(NeuralInference, ABC):
             self._build_neural_net = utils.posterior_nn(model=density_estimator)
         else:
             self._build_neural_net = density_estimator
-        self._posterior = None
-        self._sample_with_mcmc = sample_with_mcmc
-        self._mcmc_method = mcmc_method
-        self._mcmc_parameters = mcmc_parameters
-        self._rejection_sampling_parameters = rejection_sampling_parameters
 
-        self._model_bank = []
+        self._proposal_roundwise = []
         self.use_non_atomic_loss = False
 
         # Extra SNPE-specific fields summary_writer.
         self._summary.update({"rejection_sampling_acceptance_rates": []})  # type:ignore
 
-    def __call__(
+    def append_simulations(
+        self, theta: Tensor, x: Tensor, proposal: Optional[Any] = None,
+    ) -> "PosteriorEstimator":
+        r"""
+        Store parameters and simulation outputs to use them for later training.
+
+        Data are stored as entries in lists for each type of variable (parameter/data).
+
+        Stores $\theta$, $x$, prior_masks (indicating if simulations are coming from the
+        prior or not) and an index indicating which round the batch of simulations came
+        from.
+
+        Args:
+            theta: Parameter sets.
+            x: Simulation outputs.
+            proposal: The distribution that the parameters $\theta$ were sampled from.
+                Pass `None` if the parameters were sampled from the prior. If not
+                `None`, it will trigger a different loss-function.
+
+        Returns:
+            NeuralInference object (returned so that this function is chainable).
+        """
+
+        validate_theta_and_x(theta, x)
+        self._check_proposal(proposal)
+
+        if (
+            proposal is None
+            or proposal is self._prior
+            or (
+                isinstance(proposal, RestrictedPrior) and proposal._prior is self._prior
+            )
+        ):
+            # The `_data_round_index` will later be used to infer if one should train
+            # with MLE loss or with atomic loss (see, in `train()`:
+            # self._round = max(self._data_round_index))
+            self._data_round_index.append(0)
+            self._prior_masks.append(mask_sims_from_prior(0, theta.size(0)))
+        else:
+            if not self._data_round_index:
+                # This catches a pretty specific case: if, in the first round, one
+                # passes data that does not come from the prior.
+                self._data_round_index.append(1)
+            else:
+                self._data_round_index.append(max(self._data_round_index) + 1)
+            self._prior_masks.append(mask_sims_from_prior(1, theta.size(0)))
+
+        self._theta_roundwise.append(theta)
+        self._x_roundwise.append(x)
+        self._proposal_roundwise.append(proposal)
+
+        return self
+
+    def train(
         self,
-        num_simulations: int,
-        proposal: Optional[Any] = None,
         training_batch_size: int = 50,
         learning_rate: float = 5e-4,
         validation_fraction: float = 0.1,
@@ -123,17 +148,12 @@ class PosteriorEstimator(NeuralInference, ABC):
         exclude_invalid_x: bool = True,
         discard_prior_samples: bool = False,
         retrain_from_scratch_each_round: bool = False,
+        show_train_summary: bool = False,
     ) -> DirectPosterior:
-        r"""Run SNPE.
-
-        Return posterior $p(\theta|x)$ after inference.
+        r"""
+        Return density estimator that approximates the distribution $p(\theta|x)$.
 
         Args:
-            num_simulations: Number of simulator calls.
-            proposal: Distribution that the parameters $\theta$ are drawn from.
-                `proposal=None` uses the prior. Setting the proposal to a distribution
-                targeted on a specific observation, e.g. a posterior $p(\theta|x_o)$
-                obtained previously, can lead to less required simulations.
             training_batch_size: Training batch size.
             learning_rate: Learning rate for Adam optimizer.
             validation_fraction: The fraction of data to use for validation.
@@ -153,9 +173,11 @@ class PosteriorEstimator(NeuralInference, ABC):
                 samples.
             retrain_from_scratch_each_round: Whether to retrain the conditional density
                 estimator for the posterior from scratch each round.
+            show_train_summary: Whether to print the number of epochs and validation
+                loss after the training.
 
         Returns:
-            Posterior $p(\theta|x)$ that can be sampled and evaluated.
+            Density estimator that approximates the distribution $p(\theta|x)$.
         """
 
         # Calibration kernels proposed in Lueckmann, Gonçalves et al., 2017.
@@ -164,132 +186,33 @@ class PosteriorEstimator(NeuralInference, ABC):
 
         max_num_epochs = 2 ** 31 - 1 if max_num_epochs is None else max_num_epochs
 
-        self._check_proposal(proposal)
-        self._round = self._round + 1 if (proposal is not None) else 0
-
-        # If presimulated data was provided from a later round, set the self._round to
-        # this value. Otherwise, we would rely on the user to _additionally_ provide the
-        # proposal that the presimulated data was sampled from in order for self._round
-        # to become larger than 0.
-        if self._data_round_index:
-            self._round = max(self._round, max(self._data_round_index))
-
-        # Run simulations for the round.
-        theta, x = self._run_simulations(proposal, num_simulations)
-        self._append_to_data_bank(theta, x, self._round)
-
         # Load data from most recent round.
-        theta, x, _ = self._get_from_data_bank(self._round, exclude_invalid_x, False)
+        theta, x, _ = self.get_simulations(self._round, exclude_invalid_x, False)
 
         # First round or if retraining from scratch:
         # Call the `self._build_neural_net` with the rounds' thetas and xs as
         # arguments, which will build the neural network.
         # This is passed into NeuralPosterior, to create a neural posterior which
         # can `sample()` and `log_prob()`. The network is accessible via `.net`.
-        if self._posterior is None or retrain_from_scratch_each_round:
-            x_shape = x_shape_from_simulation(x)
-            self._posterior = DirectPosterior(
-                method_family="snpe",
-                neural_net=self._build_neural_net(theta, x),
-                prior=self._prior,
-                x_shape=x_shape,
-                sample_with_mcmc=self._sample_with_mcmc,
-                mcmc_method=self._mcmc_method,
-                mcmc_parameters=self._mcmc_parameters,
-                rejection_sampling_parameters=self._rejection_sampling_parameters,
-            )
-            test_posterior_net_for_multi_d_x(self._posterior.net, theta, x)
-
-        # Copy MCMC init parameters for latest sample init
-        if hasattr(proposal, "_mcmc_init_params"):
-            self._posterior._mcmc_init_params = proposal._mcmc_init_params
-
-        # Fit posterior using newly aggregated data set.
-        self._train(
-            proposal=proposal,
-            training_batch_size=training_batch_size,
-            learning_rate=learning_rate,
-            validation_fraction=validation_fraction,
-            stop_after_epochs=stop_after_epochs,
-            max_num_epochs=cast(int, max_num_epochs),
-            clip_max_norm=clip_max_norm,
-            calibration_kernel=calibration_kernel,
-            exclude_invalid_x=exclude_invalid_x,
-            discard_prior_samples=discard_prior_samples,
-        )
-
-        # Store models at end of each round.
-        self._model_bank.append(deepcopy(self._posterior))
-        self._model_bank[-1].net.eval()
-
-        # Making the call to `leakage_correction()` and the update of
-        # self._leakage_density_correction_factor explicit here. This is just
-        # to make sure this update never gets lost when we e.g. do not log our
-        # things to tensorboard anymore. Calling `leakage_correction()` is needed
-        # to update the leakage after each round.
-        if self._posterior.default_x is None:
-            acceptance_rate = torch.tensor(float("nan"))
-        else:
-            acceptance_rate = self._posterior.leakage_correction(
-                x=self._posterior.default_x,
-                force_update=True,
-                show_progress_bars=self._show_progress_bars,
-            )
-
-        # Update tensorboard and summary dict.
-        self._summarize(
-            round_=self._round,
-            x_o=self._posterior.default_x,
-            theta_bank=theta,
-            x_bank=x,
-            posterior_samples_acceptance_rate=acceptance_rate,
-        )
-
-        # Update description for progress bar.
-        if self._show_round_summary:
-            print(self._describe_round(self._round, self._summary))
-
-        self._posterior._num_trained_rounds = self._round + 1
-        return deepcopy(self._posterior)
-
-    @abstractmethod
-    def _log_prob_proposal_posterior(
-        self, theta: Tensor, x: Tensor, masks: Tensor, proposal: Optional[Any]
-    ) -> Tensor:
-        raise NotImplementedError
-
-    def _train(
-        self,
-        proposal: Optional[Any],
-        training_batch_size: int,
-        learning_rate: float,
-        validation_fraction: float,
-        stop_after_epochs: int,
-        max_num_epochs: int,
-        clip_max_norm: Optional[float],
-        calibration_kernel: Callable,
-        exclude_invalid_x: bool,
-        discard_prior_samples: bool,
-    ) -> None:
-        r"""Train the conditional density estimator for the posterior $p(\theta|x)$.
-
-        Update the conditional density estimator weights to maximize the proposal
-        posterior using the most recently aggregated bank of $(\theta, x)$ pairs.
-
-        Uses performance on a held-out validation set as a terminating condition (early
-        stopping).
-
-        The proposal is only needed for non-atomic SNPE.
-        """
+        if self._neural_net is None or retrain_from_scratch_each_round:
+            self._neural_net = self._build_neural_net(theta, x)
+            test_posterior_net_for_multi_d_x(self._neural_net, theta, x)
+            self._x_shape = x_shape_from_simulation(x)
 
         # Starting index for the training set (1 = discard round-0 samples).
         start_idx = int(discard_prior_samples and self._round > 0)
 
-        # For non-atomic loss, we can not reuse samples from previous rounds as of now.
+        # For non-atomic loss, we can not reuse samples from prev. rounds as of now.
         if self.use_non_atomic_loss:
             start_idx = self._round
 
-        theta, x, prior_masks = self._get_from_data_bank(start_idx, exclude_invalid_x)
+        theta, x, prior_masks = self.get_simulations(start_idx, exclude_invalid_x)
+
+        # Set the proposal to the last proposal that was passed by the user. For
+        # atomic SNPE, it does not matter what the proposal is. For non-atomic
+        # SNPE, we only use the latest data that was passed, i.e. the one from the
+        # last proposal.
+        proposal = self._proposal_roundwise[-1]
 
         # Select random neural net and validation splits from (theta, x) pairs.
         num_total_examples = len(theta)
@@ -319,15 +242,13 @@ class PosteriorEstimator(NeuralInference, ABC):
             sampler=SubsetRandomSampler(val_indices),
         )
 
-        optimizer = optim.Adam(
-            list(self._posterior.net.parameters()), lr=learning_rate,
-        )
+        optimizer = optim.Adam(list(self._neural_net.parameters()), lr=learning_rate,)
 
         epoch, self._val_log_prob = 0, float("-Inf")
         while epoch <= max_num_epochs and not self._converged(epoch, stop_after_epochs):
 
             # Train for a single epoch.
-            self._posterior.net.train()
+            self._neural_net.train()
             for batch in train_loader:
                 optimizer.zero_grad()
                 theta_batch, x_batch, masks_batch = (
@@ -338,20 +259,20 @@ class PosteriorEstimator(NeuralInference, ABC):
 
                 batch_loss = torch.mean(
                     self._loss(
-                        theta_batch, x_batch, masks_batch, proposal, calibration_kernel
+                        theta_batch, x_batch, masks_batch, proposal, calibration_kernel,
                     )
                 )
                 batch_loss.backward()
                 if clip_max_norm is not None:
                     clip_grad_norm_(
-                        self._posterior.net.parameters(), max_norm=clip_max_norm,
+                        self._neural_net.parameters(), max_norm=clip_max_norm,
                     )
                 optimizer.step()
 
             epoch += 1
 
             # Calculate validation performance.
-            self._posterior.net.eval()
+            self._neural_net.eval()
             log_prob_sum = 0
             with torch.no_grad():
                 for batch in val_loader:
@@ -362,11 +283,13 @@ class PosteriorEstimator(NeuralInference, ABC):
                     )
                     # Take negative loss here to get validation log_prob.
                     batch_log_prob = -self._loss(
-                        theta_batch, x_batch, masks_batch, proposal, calibration_kernel
+                        theta_batch, x_batch, masks_batch, proposal, calibration_kernel,
                     )
                     log_prob_sum += batch_log_prob.sum().item()
 
             self._val_log_prob = log_prob_sum / num_validation_examples
+            # Log validation log prob for every epoch.
+            self._summary["validation_log_probs"].append(self._val_log_prob)
 
             self._maybe_show_progress(self._show_progress_bars, epoch)
 
@@ -376,6 +299,92 @@ class PosteriorEstimator(NeuralInference, ABC):
         self._summary["epochs"].append(epoch)
         self._summary["best_validation_log_probs"].append(self._best_val_log_prob)
 
+        # Update tensorboard and summary dict.
+        self._summarize(
+            round_=self._round, x_o=None, theta_bank=theta, x_bank=x,
+        )
+
+        # Update description for progress bar.
+        if show_train_summary:
+            print(self._describe_round(self._round, self._summary))
+
+        return deepcopy(self._neural_net)
+
+    def build_posterior(
+        self,
+        density_estimator: Optional[TorchModule] = None,
+        rejection_sampling_parameters: Optional[Dict[str, Any]] = None,
+        sample_with_mcmc: bool = False,
+        mcmc_method: str = "slice_np",
+        mcmc_parameters: Optional[Dict[str, Any]] = None,
+    ) -> DirectPosterior:
+        r"""
+        Build posterior from the neural density estimator.
+
+        For SNPE, the posterior distribution that is returned here implements the
+        following functionality over the raw neural density estimator:
+
+        - correct the calculation of the log probability such that it compensates for
+            the leakage.
+        - reject samples that lie outside of the prior bounds.
+        - alternatively, if leakage is very high (which can happen for multi-round
+            SNPE), sample from the posterior with MCMC.
+
+        Args:
+            density_estimator: The density estimator that the posterior is based on.
+                If `None`, use the latest neural density estimator that was trained.
+            rejection_sampling_parameters: Dictionary overriding the default parameters
+                for rejection sampling. The following parameters are supported:
+                `max_sampling_batch_size` to set the batch size for drawing new
+                samples from the candidate distribution, e.g., the posterior. Larger
+                batch size speeds up sampling.
+            sample_with_mcmc: Whether to sample with MCMC. MCMC can be used to deal
+                with high leakage.
+            mcmc_method: Method used for MCMC sampling, one of `slice_np`, `slice`,
+                `hmc`, `nuts`. Currently defaults to `slice_np` for a custom numpy
+                implementation of slice sampling; select `hmc`, `nuts` or `slice` for
+                Pyro-based sampling.
+            mcmc_parameters: Dictionary overriding the default parameters for MCMC.
+                The following parameters are supported: `thin` to set the thinning
+                factor for the chain, `warmup_steps` to set the initial number of
+                samples to discard, `num_chains` for the number of chains,
+                `init_strategy` for the initialisation strategy for chains; `prior` will
+                draw init locations from prior, whereas `sir` will use
+                Sequential-Importance-Resampling using `init_strategy_num_candidates`
+                to find init locations.
+
+        Returns:
+            Posterior $p(\theta|x)$  with `.sample()` and `.log_prob()` methods.
+        """
+
+        if density_estimator is None:
+            density_estimator = self._neural_net
+
+        self._posterior = DirectPosterior(
+            method_family="snpe",
+            neural_net=density_estimator,
+            prior=self._prior,
+            x_shape=self._x_shape,
+            rejection_sampling_parameters=rejection_sampling_parameters,
+            sample_with_mcmc=sample_with_mcmc,
+            mcmc_method=mcmc_method,
+            mcmc_parameters=mcmc_parameters,
+        )
+
+        self._posterior._num_trained_rounds = self._round + 1
+
+        # Store models at end of each round.
+        self._model_bank.append(deepcopy(self._posterior))
+        self._model_bank[-1].net.eval()
+
+        return deepcopy(self._posterior)
+
+    @abstractmethod
+    def _log_prob_proposal_posterior(
+        self, theta: Tensor, x: Tensor, masks: Tensor, proposal: Optional[Any]
+    ) -> Tensor:
+        raise NotImplementedError
+
     def _loss(
         self,
         theta: Tensor,
@@ -395,8 +404,44 @@ class PosteriorEstimator(NeuralInference, ABC):
 
         if self._round == 0:
             # Use posterior log prob (without proposal correction) for first round.
-            log_prob = self._posterior.net.log_prob(theta, x)
+            log_prob = self._neural_net.log_prob(theta, x)
         else:
             log_prob = self._log_prob_proposal_posterior(theta, x, masks, proposal)
 
         return -(calibration_kernel(x) * log_prob)
+
+    def _check_proposal(self, proposal):
+        """
+        Check for validity of the provided proposal distribution.
+        If the proposal is a `NeuralPosterior`, we check if the default_x is set.
+        If the proposal is **not** a `NeuralPosterior`, we warn since it is likely that
+        the user simply passed the prior, but this would still trigger atomic loss.
+        """
+        if proposal is not None:
+            check_if_proposal_has_default_x(proposal)
+
+            if isinstance(proposal, RestrictedPrior):
+                if proposal._prior is not self._prior:
+                    warn(
+                        "The proposal you passed is a `RestrictedPrior`, but the "
+                        "proposal distribution it uses is not the prior (it can be "
+                        "accessed via `RestrictedPrior._prior`). We do not "
+                        "recommend to mix the `RestrictedPrior` with multi-round "
+                        "SNPE."
+                    )
+            elif (
+                not isinstance(proposal, DirectPosterior)
+                and proposal is not self._prior
+            ):
+                warn(
+                    "The proposal you passed is neither the prior nor a "
+                    "`NeuralPosterior` object. If you are an expert user and did so "
+                    "for research purposes, this is fine. If not, you might be doing "
+                    "something wrong: feel free to create an issue on Github."
+                )
+        elif self._round > 0:
+            raise ValueError(
+                "You did not specify a proposal (i.e. `proposal=None`). "
+                "However, previously, you had already specified a proposal. "
+                "This scenario is currently not allowed."
+            )
